@@ -47,7 +47,7 @@ void remove_wrong_password_list(t_config *config);
 
 void close_client_sockets_for_cgi_run(void);
 
-#ifdef ENABLE_COMMAND
+#ifdef ENABLE_TOMAHAWK
 void print_client_list(FILE *fp);
 int  number_of_clients(void);
 void print_ban_list(FILE *fp);
