@@ -1891,7 +1891,7 @@ int run_server(t_settings *settings) {
 
 	/* Setup poll data
 	 */
-	if ((poll_data = (struct pollfd*)malloc((number_of_bindings + MAX_ADMIN_CONNECTIONS) * sizeof(struct pollfd*))) == NULL) {
+	if ((poll_data = (struct pollfd*)malloc((number_of_bindings + MAX_ADMIN_CONNECTIONS) * sizeof(struct pollfd))) == NULL) {
 		return -1;
 	}
 
