@@ -13,15 +13,20 @@
 #define _TARGET_H
 
 #include "session.h"
+#ifdef ENABLE_RPROXY
+#include "rproxy.h"
+#endif
 
 int send_file(t_session *session);
 int execute_cgi(t_session *session);
-//int handle_options_request(t_session *session);
 int handle_trace_request(t_session *session);
 int handle_put_request(t_session *session);
 int handle_delete_request(t_session *session);
 #ifdef ENABLE_XSLT
 int handle_xml_file(t_session *session);
 #endif
+#ifdef ENABLE_RPROXY
+int proxy_request(t_session *session, t_rproxy *proxy);
+#endif
 
 #endif
